import boto3


class SNS(object):
    """
    This class is to process the notifications by using SNS service
    """
    def __init__(self, region=''):
        if region:
            self._sns = boto3.client('sns', region_name=region)
        else:
            self._sns = boto3.client('sns')

    def publish(self, topic_arn, subject, message):
        """
        Publish messages to SNS
        """
        return self._sns.publish(TopicArn=topic_arn,
                                 Subject=subject,
                                 Message=message)

        


import boto3
from datetime import datetime
import json
import uuid


class StandardSQS(object):
    """
    This class is to process the messages by using standard SQS service
    """
    def __init__(self, queue_url):
        self._queue = boto3.resource('sqs').Queue(queue_url)

    def send_message(self, message_body, delay=0, message_attributes=None):
        """
        Sending messages to SQS
        """
        return self._queue.send_message(MessageBody=message_body,
                                        DelaySeconds=delay,
                                        MessageAttributes=message_attributes)

    def receive_messages(self, delete_messages=True, latency_in_seconds=0):
        """
        Retrieving messages could be tricky, since the number of messages to
        be returned is not guaranteed. Sometimes the network latency will
        cause the failure of messages retrieving attempts.

        :return: a generator of messages from SQS
        """
        gen = (message for message in
               self._queue.receive_messages(MessageAttributeNames=['All'],
                                            MaxNumberOfMessages=10,
                                            WaitTimeSeconds=latency_in_seconds))

        for item in gen:
            yield item
            if delete_messages:
                item.delete()


class MonitorMessage(object):
    """
    This should be a utility class acts like a monitor message
    template generator.
    """
    @staticmethod
    def get_default_message(client_id, asset_manager_id, message,
                            item_id=None, item_level=None, item_status=None,
                            item_class=None, item_type=None,
                            item_source=None, version=None, item_date=None, action='Insert'):
        """
        This will generate the default Monitor message template to be sent to
        SQS
        """
        if not client_id or not asset_manager_id:
            return False
        else:
            message = {"client_id": client_id,
                       "action": action,
                       "content": {"asset_manager_id": asset_manager_id,
                                   "item_id": item_id or uuid.uuid4().hex,
                                   "item_level": item_level or 'Error',
                                   "item_status": item_status or 'Open',
                                   "item_class": item_class or 'Exception',
                                   "item_type": item_type or '',
                                   "item_source": item_source or 'TEMP',
                                   "version": version or 1,
                                   "message": message,
                                   "item_date": item_date or str(
                                       datetime.now())}}

            return json.dumps(message)



import redis

class Cache(object):
    def __init__(self, server='localhost', port=6379, db=0):
        redis_db = redis.StrictRedis(host=server, port=port, db=db)

    def set(self, key, value):
        raise NotImplementedError()

    def get(self, key):
        raise NotImplementedError()


class Redis(Cache):
    def __init__(self):
        super().__init__()

    def set(self, key, value):
        pass

    def get(self, key):
        pass


import click
import os
import pymysql
from configparser import ConfigParser, NoSectionError
from config.amaas_config import AMaaSConfig

class DatabaseSetter(object):
    def __init__(self):
        pass

    def recreate(self, schema, environment):
        environment = environment.lower()
        config = AMaaSConfig()
        if environment in ['dev', 'staging']:
            click.echo(f"Using {environment} Settings") 
            server = config.get_config_value(environment, 'db_server')
            username = config.get_config_value(environment, 'db_username')
            password = config.get_config_value(environment, 'db_password')
        else:
            click.echo("Using Local Settings")
            server="localhost"
            username="root"
            password="amaas"
        #connect to the server and create database
        try:
            click.echo("Connecting to the server")
            conn = pymysql.connect(server, user=username, passwd=password, charset='utf8')
            conn.cursor().execute("SET time_zone = '+00:00';")
            click.echo("Creating schema: " + schema)
            conn.cursor().execute("DROP DATABASE IF EXISTS "+ schema +";")
            conn.cursor().execute("CREATE DATABASE "+schema+" CHARACTER SET utf8 COLLATE utf8_unicode_ci;")
            conn.cursor().execute("USE "+schema+";")
            click.echo("Schema " + schema + " created successfully.")
        except (pymysql.InternalError, pymysql.OperationalError):
            raise Exception('Error Creating DB and Connection') 
        tables_path = os.path.join(os.getcwd() + "/tables")
        files = [f for f in os.listdir(tables_path)]
        #executing sql files
        try:
            for file in files:
                file_path = os.path.join(tables_path, file)
                with open(file_path) as SQL_file:
                    content=SQL_file.readlines()
                exe_line = ''
                for line in content:
                    exe_line+=line.partition('#')[0].rstrip()
                    if exe_line!='' and str(exe_line[-1]) == ';':
                        print("executing: "+exe_line)
                        conn.cursor().execute(exe_line)
                        exe_line=''
                click.echo("Create table "+file[:-4]+" successfully")
        except (pymysql.OperationalError):
            raise Exception('Error executing mysql files in tables')

if __name__ == '__main__':
    DatabaseSetter().recreate('asset_managers', 'environment')


"""
This module is used to manage transaction emails
"""
import boto3
import json
import requests


class EmailClient(object):
    """
    This is a template class that all email clients should inherite from
    """
    def __init__(self):
        pass

    def send_mail(self, **kwargs):
        """
        This method should be implemented by every email client
        """
        raise NotImplementedError()


class SESClient(EmailClient):
    """
    This is an implementation of SES
    """
    def __init__(self, region='us-west-2'):
        self.client = boto3.client('ses', region_name=region)

    def send_mail(self, email):
        """
        Implementation of SES sending email
        """
        if isinstance(email, SimpleEmail):
            return self.client.send_email(Source=email.sender,
                                          Destination={'ToAddresses': [email.to]},
                                          Message={'Subject': {'Data': email.subject,
                                                               'Charset': 'utf8'},
                                                   'Body': {'Text': {'Data': email.message,
                                                                     'Charset': 'utf8'},
                                                            'Html': {'Data': email.subject,
                                                                     'Charset': 'utf8'}}},
                                          ReplyToAddresses=[email.sender])
        else:
            raise TypeError('Invalid parameter type, SimpleEmail object required.')


class SimpleEmail(object):
    def __init__(self, subject, message, to, sender='noreply@argomi.com'):
        self._sender= sender
        self._to = to
        self._subject = subject
        self._message = message

    @property
    def sender(self):
        return self._sender

    @property
    def to(self):
        return self._to

    @property
    def subject(self):
        return self._subject

    @property
    def message(self):
        return self._message

if __name__ == '__main__':
    SESClient().send_mail(SimpleEmail(subject='test email', message='test message', to='cheng.chen@amaas.com'))


""" Logs calls to the AMaaS APIs """

from datetime import datetime

import boto3

class APICallLogger(object):
    """ The current implementation of the API Call Logger uses Cloudwatch metrics """

    def __init__(self):
        self.client = boto3.client('cloudwatch')

    def record_api_call(self, asset_manager_id, api_call_type, api_call_subtype, environment='dev'):
        """ Record the API call """
        asset_manager_id = str(asset_manager_id)  # In case it gets sent in as an integer
        self.client.put_metric_data(
            Namespace='amaas',
            MetricData=[
                {
                    'MetricName': 'AMaaSAPICalls',
                    'Dimensions': [
                        {'Name': 'api_call_type', 'Value': api_call_type},
                        {'Name': 'api_call_subtype', 'Value': api_call_subtype},
                        {'Name': 'asset_manager_id', 'Value': asset_manager_id},
                        {'Name': 'environment', 'Value': environment}
                    ],
                    'Timestamp': datetime.utcnow(),
                    'Value': 1,
                    'Unit': 'Count'
                },
            ]
        )
